/***************************************************************************
                          board.h  -  description
                             -------------------
    begin                : Thu Aug 17 2000
    copyright            : (C) 2000 by Waldemar Baraldi
    email                : baraldi@lacasilla.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include "object.h"
#include "list.h"
#include "canvas.h"
#include "pipe.h"
#include "pointer.h"
#include "entry.h"
#include "exit.h"
#include "tank.h"
#include "animatedcanvas.h"

enum BoardState {Playing, OverLost, OverWon};

class Board: public AnimatedCanvas{

  public:

    static const int rows=8;
    static const int columns=9;
    static const int TankX=90;
    static const int TankY=0;

    Board();
    virtual ~Board();

    int width();

    int height();

    /** Setea la cantidad de ticks hasta comenzar
    a avanzar el flujo.*/
    void setStartDelay(int delay);

    /** Asigna la entrada del flujo.*/
    //void setEntry(Entry * entry, int row, int column);
    void setEntry(Entry * entry, int row, int column);

    /** Asigna la salida del flujo.*/
    //void setExit(Exit * exit, int row, int column);
    void setExit(Exit * exit, int row, int column);

    /** Retorna true si el pipe en row, column es removible.*/
    bool isRemovable(int row, int column);

    /** Asigna pipe a la posicion row,column. Si las coordenadas son
        invalidas, el resultado es indefinido. pipe no debe ser NULL.*/
    void setPipe(Pipe * pipe, int row, int column);

    /** Retorna el pipe en la posicion. Si las coordenadas no son
        validas, el resultado es indefinido.*/
    Pipe * getPipe(int row, int column);


    /** Retorna el estado del board.*/
    BoardState getState();

    /** Agrega el puntero en la posicion que tenga.*/
    void addPointer(Pointer * p);

    /** Quita el puntero del board.*/
    void removePointer(Pointer * p);

   /** Mueve el puntero de posicion a row, column.*/
    void movePointer(Pointer *p, int row, int column);

    void setPos(int x, int y);

    void setSpeed(int amount);

    void setMaxSpeed();

    void setRestrictionCoef(unsigned int coef);

    void setFixedCoef(unsigned int coef);

    unsigned int getRestrictionCoef();

    unsigned int getFixedCoef();

    /** Sealiza el avance del lquido.*/
    void tick();

    void paint(VideoManager * vm);

  protected:

    List * lps;

    Pipe * board[rows][columns];

    Tank * tank;

    bool changes[rows][columns];
    bool paint_all;

    Image * background;

    int flow_row, flow_column;

    Pipe * entry;
    Pipe * exit;

    int start_delay;
    int tick_amount;

    CardinalPoint actual_output;
    CardinalPoint actual_input;

    unsigned int rest_coef;
    unsigned int fixed_coef;

    BoardState state;
};

#endif
