/***************************************************************************
                          pipenightdreams.cpp  -  description
                             -------------------
    begin                : Thu Aug 17 2000
    copyright            : (C) 2000 by Waldemar Baraldi
    email                : baraldi@lacasilla.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pipenightdreams.h"
#include "SDL.h"
#include "str.h"
#include "imagemanager.h"
#include "eventmanager.h"
#include "playeronestream.h"
#include "level.h"

PipeNightDreams::PipeNightDreams(){
  SDL_Init(SDL_INIT_VIDEO);

  video_manager=new VideoManager();
  event_manager=new EventManager();
  sys = new SystemStream();
  event_manager->addStream(sys);
  data_dir=new Str();
  images_dir=new Str();
}

PipeNightDreams::~PipeNightDreams(){
  delete data_dir;
  delete images_dir;
  delete video_manager;
  delete event_manager;
//  delete main;
  delete nextlevel;
  delete tryagain;
//  delete gameover;
  SDL_Quit();
}

void PipeNightDreams::setDataDir(Str * dir_name){
  delete data_dir;
  delete images_dir;
  data_dir=dir_name;

  images_dir=new Str(data_dir);
  images_dir->concat("/images/");
  video_manager->getImageManager()->setBaseDir(new Str(images_dir));
}

void PipeNightDreams::run(){
  quit=false;
  load();
  showMainMenu();
  while (!quit){
    PlayerOneStream * pone = new PlayerOneStream();
    Player * player= new Player(pone);
    bool game_over=false;
    int l=1;
    BoardState bs;

    event_manager->addStream(pone);
    event_manager->pumpEvents();

    Board * board=new Board();
    while (!game_over){
      board->setPos(100,0);
      playLevel(l, board, player);
      bs=board->getState();
      delete board;
      if (bs==Playing)
        game_over=true;
      else{
        sys->clear();
        switch (bs){
          case OverLost:{
            if (player->lives()>0){
              player->decLives();
              SDL_Delay(750);
              showTryAgain();
              board=new Board();
            }else{
              game_over=true;
              (video_manager->getImageManager())->empty();
              showGameOver();
            }
            break;
          }
          case OverWon:{
            (video_manager->getImageManager())->empty();
            showNextLevel();
            l++; //Incremento el nivel
            board=new Board();
            break;
          }
          default:break;
        }
      }
    }
    delete player;
    showMainMenu();
  }
}

void PipeNightDreams::showIntro(){
}

void PipeNightDreams::showOptions(){
}

void PipeNightDreams::playLevel(unsigned int level_number, Board * board, Player * player){
  char aux[128];
  Str * str= new Str(data_dir);
  Event event;
  sprintf(aux, "/levels/%i.level", level_number);
  str->concat(aux);
  Level * level=new Level();
  if (!level->load(board,video_manager->getImageManager(), str)){
    int c,d;
    player->setBoard(board);
    while (event!=BACK && board->getState()==Playing){
      c=SDL_GetTicks();
      event=sys->get();
      player->play();
      board->tick();
      board->paint(video_manager);
      if (player->isChanged()) player->paint(video_manager);
      video_manager->flush();
      event_manager->pumpEvents();
      d=SDL_GetTicks();
      if (d-c<(int)(1000/FPS)) SDL_Delay((int)(1000/FPS)-(d-c));
    }
  }else{
    printf("All levels done!!!!!!!!!!!\n");
    game_over=true;
  }
  delete level;
}

void PipeNightDreams::showMainMenu(){
  Event event;
  Str * aux;
  Image * main;

  aux= new Str(images_dir);
  aux->concat("start.png");
  main=new Image(aux);

  video_manager->blit(main, 0,0);
  video_manager->flush();
  delete main;

  do{
    event_manager->pumpEvents(true);
    event=sys->get();
  }while (event!=GO && event!=BACK);
  if (event==BACK) quit=true;
}

void PipeNightDreams::showGameOver(){
  Event event;
  Image * gameover;
  Str * aux;

  aux= new Str(images_dir);
  aux->concat("gameover.png");
  gameover=new Image(aux);

  video_manager->blit(gameover, 100+270-gameover->width()/2,240-gameover->height()/2);
  video_manager->flush();
  delete gameover;

  do{
    event_manager->pumpEvents(true);
    event=sys->get();
  }while (event!=BACK);
}

void PipeNightDreams::showHallOfFame(){
}

void PipeNightDreams::showNextLevel(){
  Event event;

  video_manager->blit(nextlevel, 100+270-nextlevel->width()/2,240-nextlevel->height()/2);
  video_manager->flush();

  do{
    event_manager->pumpEvents(true);
    event=sys->get();
  }while (event!=BACK);
}

void PipeNightDreams::showTryAgain(){
  Event event;
  video_manager->blit(tryagain, 100+270-tryagain->width()/2,240-tryagain->height()/2);
  video_manager->flush();

  do{
    event_manager->pumpEvents(true);
    event=sys->get();
  }while (event!=GO);
}

void PipeNightDreams::load(){
  Image * loading;
  Str * aux;

  aux= new Str(images_dir);
  aux->concat("loading.png");
  loading=new Image(aux);
  video_manager->blit(loading, 320-loading->width()/2,240-loading->height()/2);
  video_manager->flush();
  delete loading;

  aux= new Str(images_dir);
  aux->concat("tryagain.png");
  tryagain=new Image(aux);

  aux= new Str(images_dir);
  aux->concat("nextlevel.png");
  nextlevel=new Image(aux);
}


