/***************************************************************************
                          player.cpp  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (C) 2000 by Waldemar Baraldi
    email                : baraldi@lacasilla.com.ar
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "player.h"

#define MAX_LIVES 8

Player::Player(EventStream * es):AnimatedCanvas(){
  stream=es;
  queue=NULL;
  score=0;
  start_row=0;
  start_column=0;
  change=true;
  nlives=StartLives;
}

Player::~Player(){
  if (queue) delete queue;
}

void Player::setPos(int x, int y){
  Canvas::setPos(x,y);
  queue->setPos((width()-queue->width())/2, y);
}

void Player::setStartRowColumn(int row, int column){
  start_row=row;
  start_column=column;
}

void Player::setBoard(Board * bd){
  board=bd;

  if (queue) delete queue;
  queue=new PipeQueue();
  queue->setPos(x+27, y+5);
  queue->setRestrictionCoef(board->getRestrictionCoef());
  queue->setFixedCoef(board->getFixedCoef());
  queue->fillUp();

  pointer=queue->getPipe(0)->getPointer();
  board->addPointer(pointer);
  board->movePointer(pointer,start_row,start_column);

  stream->clear();
}

void Player::incLives(){
  nlives++;
  change=true;
}

void Player::decLives(){
  if (nlives>0) nlives--;
  change=true;
}

unsigned int Player::lives(){
  return nlives;
}

bool Player::isChanged(){
  return change || queue->isChanged();
}

void Player::play(){
  Event event;

  event=stream->get();
  switch (event){
    case UP:{
      if (pointer->getRow()> 0)
        board->movePointer(pointer, pointer->getRow()-1, pointer->getColumn());
      break;
    }
    case DOWN:{
      if (pointer->getRow()<board->rows-1)
        board->movePointer(pointer, pointer->getRow()+1, pointer->getColumn());
      break;
    }
    case LEFT:{
      if (pointer->getColumn()>0)
        board->movePointer(pointer, pointer->getRow(), pointer->getColumn()-1);
      break;
    }
    case RIGHT:{
      if (pointer->getColumn()<board->columns-1)
                board->movePointer(pointer, pointer->getRow(), pointer->getColumn()+1);
      break;
    }
    case HIT:{
      int r=pointer->getRow(),c=pointer->getColumn();
      if (board->isRemovable(r,c)){
        board->removePointer(pointer);
        Pipe * aux = queue->getHead();
        board->setPipe(aux, r, c);
        pointer=queue->getPipe(0)->getPointer();
        board->movePointer(pointer,r, c);
        board->addPointer(pointer);
      }
      else{
//        printf("Ups!\n");
      }
      break;
    }
    case GO:{
      board->setMaxSpeed();
    }
    default:break;
  }
}

void Player::paint(VideoManager * vm){

  //Pintar el background
  if (change){
    vm->fillRect(x+LivesX,y+LivesY,MAX_LIVES*LivesWidth,LivesHeight, 0,0,0,255);
    //Dibuja el liquido
    vm->fillRect(x+LivesX,y+LivesY, lives()*LivesWidth, LivesHeight, 103,234,16,255);

    Image * ima=vm->getImageManager()->getImage(new Str("panel.png"));
    vm->blit(ima, x,y);
    change=false;
  }
  //Pintar la cola
    queue->paint(vm);
}

